import 'dart:async';
import 'dart:convert';
import 'dart:io';
import '../consts.dart';
import 'dart:typed_data';
import 'dart:io';
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';

/// Fetch relation data from server
Future<Uint8List> fetchRelation() async {
  try {
    // Custom HTTP client
    HttpClient client = HttpClient()
      ..badCertificateCallback = // NB: temporary disable SSL certificate validation
          (X509Certificate cert, String host, int port) => true;

    // Execute request to to get_relation endpoint
    var request = await client.getUrl(Uri.parse('${serverURI}get_relation'));
    var response = await request.close(); // Close response body at end of function

    // Try to parse body to JSON if request is ok
    if (response.statusCode == 200) {
      var responseBody = await response.transform(utf8.decoder).join();

      if (responseBody.isNotEmpty) {
        Directory appDocumentsDirectory = await getApplicationDocumentsDirectory();
        String filePath = '${appDocumentsDirectory.path}/last_update.txt';

        try { // Write most recent time of update to file
          await File(filePath).writeAsString('${DateTime.now()}', mode: FileMode.write);
          print('Update time written to file');
        } catch (error) { print('Error in writing to file: $error');}

        // Return relation data from the response body
        return Uint8List.fromList(utf8.encode(responseBody));
      } else {
        throw Exception('Response body is empty');
      }
    } else {
      throw Exception('Failed to fetch relation data: Status code ${response.statusCode}');
    }
  } catch (e) {
    throw Exception('Failed to fetch relation data: ${e.toString()}');
  }
}

